// file generated by src/provider/update.py

use crate::provider::Protocol::*;
use crate::provider::Socket::*;
use crate::provider::UsernamePattern::*;
use crate::provider::*;
use std::collections::HashMap;

lazy_static::lazy_static! {

    // aktivix.org.md: aktivix.org
    static ref P_AKTIVIX_ORG: Provider = Provider {
        status: Status::OK,
        before_login_hint: "",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/aktivix-org",
        server: vec![
            Server { protocol: IMAP, socket: STARTTLS, hostname: "newyear.aktivix.org", port: 143, username_pattern: EMAIL },
            Server { protocol: SMTP, socket: STARTTLS, hostname: "newyear.aktivix.org", port: 25, username_pattern: EMAIL },
        ],
    };

    // aol.md: aol.com
    static ref P_AOL: Provider = Provider {
        status: Status::PREPARATION,
        before_login_hint: "To log in to AOL with Delta Chat, you need to set up an app password in the AOL web interface.",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/aol",
        server: vec![
        ],
    };

    // autistici.org.md: autistici.org
    static ref P_AUTISTICI_ORG: Provider = Provider {
        status: Status::OK,
        before_login_hint: "",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/autistici-org",
        server: vec![
            Server { protocol: IMAP, socket: SSL, hostname: "mail.autistici.org", port: 993, username_pattern: EMAIL },
            Server { protocol: SMTP, socket: SSL, hostname: "smtp.autistici.org", port: 465, username_pattern: EMAIL },
        ],
    };

    // bluewin.ch.md: bluewin.ch
    static ref P_BLUEWIN_CH: Provider = Provider {
        status: Status::OK,
        before_login_hint: "",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/bluewin-ch",
        server: vec![
            Server { protocol: IMAP, socket: SSL, hostname: "imaps.bluewin.ch", port: 993, username_pattern: EMAIL },
            Server { protocol: SMTP, socket: SSL, hostname: "smtpauths.bluewin.ch", port: 465, username_pattern: EMAIL },
        ],
    };

    // comcast.md: xfinity.com, comcast.net
    // - skipping provider with status OK and no special things to do

    // dismail.de.md: dismail.de
    // - skipping provider with status OK and no special things to do

    // disroot.md: disroot.org
    // - skipping provider with status OK and no special things to do

    // example.com.md: example.com, example.org
    static ref P_EXAMPLE_COM: Provider = Provider {
        status: Status::BROKEN,
        before_login_hint: "Hush this provider doesn't exist!",
        after_login_hint: "This provider doesn't really exist, so you can't use it :/ If you need an email provider for Delta Chat, take a look at providers.delta.chat!",
        overview_page: "https://providers.delta.chat/example-com",
        server: vec![
            Server { protocol: IMAP, socket: SSL, hostname: "imap.example.com", port: 1337, username_pattern: EMAIL },
            Server { protocol: SMTP, socket: STARTTLS, hostname: "smtp.example.com", port: 1337, username_pattern: EMAIL },
        ],
    };

    // fastmail.md: fastmail.com
    static ref P_FASTMAIL: Provider = Provider {
        status: Status::PREPARATION,
        before_login_hint: "You must create an app-specific password for Delta Chat before you can log in.",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/fastmail",
        server: vec![
        ],
    };

    // freenet.de.md: freenet.de
    static ref P_FREENET_DE: Provider = Provider {
        status: Status::OK,
        before_login_hint: "",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/freenet-de",
        server: vec![
            Server { protocol: IMAP, socket: SSL, hostname: "mx.freenet.de", port: 993, username_pattern: EMAIL },
            Server { protocol: SMTP, socket: STARTTLS, hostname: "mx.freenet.de", port: 587, username_pattern: EMAIL },
        ],
    };

    // gmail.md: gmail.com, googlemail.com
    static ref P_GMAIL: Provider = Provider {
        status: Status::PREPARATION,
        before_login_hint: "For Gmail accounts, you need to create an app-password if you have \"2-Step Verification\" enabled. If this setting is not available, you need to enable \"less secure apps\".",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/gmail",
        server: vec![
            Server { protocol: IMAP, socket: SSL, hostname: "imap.gmail.com", port: 993, username_pattern: EMAIL },
            Server { protocol: SMTP, socket: SSL, hostname: "smtp.gmail.com", port: 465, username_pattern: EMAIL },
        ],
    };

    // gmx.net.md: gmx.net, gmx.de, gmx.at, gmx.ch, gmx.org, gmx.eu, gmx.info, gmx.biz, gmx.com
    static ref P_GMX_NET: Provider = Provider {
        status: Status::PREPARATION,
        before_login_hint: "You must allow IMAP access to your account before you can login.",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/gmx-net",
        server: vec![
            Server { protocol: IMAP, socket: SSL, hostname: "imap.gmx.net", port: 993, username_pattern: EMAIL },
            Server { protocol: SMTP, socket: SSL, hostname: "mail.gmx.net", port: 465, username_pattern: EMAIL },
            Server { protocol: SMTP, socket: STARTTLS, hostname: "mail.gmx.net", port: 587, username_pattern: EMAIL },
        ],
    };

    // i.ua.md: i.ua
    // - skipping provider with status OK and no special things to do

    // icloud.md: icloud.com, me.com, mac.com
    static ref P_ICLOUD: Provider = Provider {
        status: Status::PREPARATION,
        before_login_hint: "You must create an app-specific password for Delta Chat before you can login.",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/icloud",
        server: vec![
            Server { protocol: IMAP, socket: SSL, hostname: "imap.mail.me.com", port: 993, username_pattern: EMAILLOCALPART },
            Server { protocol: SMTP, socket: STARTTLS, hostname: "smtp.mail.me.com", port: 587, username_pattern: EMAIL },
        ],
    };

    // kolst.com.md: kolst.com
    // - skipping provider with status OK and no special things to do

    // kontent.com.md: kontent.com
    // - skipping provider with status OK and no special things to do

    // mail.ru.md: mail.ru, inbox.ru, bk.ru, list.ru
    // - skipping provider with status OK and no special things to do

    // mailbox.org.md: mailbox.org, secure.mailbox.org
    // - skipping provider with status OK and no special things to do

    // nauta.cu.md: nauta.cu
    static ref P_NAUTA_CU: Provider = Provider {
        status: Status::OK,
        before_login_hint: "",
        after_login_hint: "Atención - con nauta.cu, puede enviar mensajes sólo a un máximo de 20 personas a la vez. En grupos más grandes, no puede enviar mensajes o abandonar el grupo.",
        overview_page: "https://providers.delta.chat/nauta-cu",
        server: vec![
            Server { protocol: IMAP, socket: STARTTLS, hostname: "imap.nauta.cu", port: 143, username_pattern: EMAIL },
            Server { protocol: SMTP, socket: STARTTLS, hostname: "smtp.nauta.cu", port: 25, username_pattern: EMAIL },
        ],
    };

    // outlook.com.md: hotmail.com, outlook.com, office365.com, outlook.com.tr, live.com
    static ref P_OUTLOOK_COM: Provider = Provider {
        status: Status::BROKEN,
        before_login_hint: "Outlook.com email addresses will not work as expected as these servers remove some important transport information. Hopefully sooner or later there will be a fix, for now we suggest to use another email address.",
        after_login_hint: "Outlook.com email addresses will not work as expected as these servers remove some important transport information. Unencrypted 1-on-1 chats kind of work, but groups and encryption don't. Hopefully sooner or later there will be a fix, for now we suggest to use another email address.",
        overview_page: "https://providers.delta.chat/outlook-com",
        server: vec![
            Server { protocol: IMAP, socket: SSL, hostname: "imap-mail.outlook.com", port: 993, username_pattern: EMAIL },
            Server { protocol: SMTP, socket: STARTTLS, hostname: "smtp-mail.outlook.com", port: 587, username_pattern: EMAIL },
        ],
    };

    // posteo.md: posteo.de
    static ref P_POSTEO: Provider = Provider {
        status: Status::OK,
        before_login_hint: "",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/posteo",
        server: vec![
            Server { protocol: IMAP, socket: STARTTLS, hostname: "posteo.de", port: 143, username_pattern: EMAIL },
            Server { protocol: SMTP, socket: STARTTLS, hostname: "posteo.de", port: 587, username_pattern: EMAIL },
        ],
    };

    // protonmail.md: protonmail.com, protonmail.ch
    static ref P_PROTONMAIL: Provider = Provider {
        status: Status::BROKEN,
        before_login_hint: "Protonmail does not offer the standard IMAP e-mail protocol, so you cannot log in with Delta Chat to Protonmail.",
        after_login_hint: "To use Delta Chat with Protonmail, the IMAP bridge must be running in the background. If you have connectivity issues, double check whether it works as expected.",
        overview_page: "https://providers.delta.chat/protonmail",
        server: vec![
        ],
    };

    // riseup.net.md: riseup.net
    // - skipping provider with status OK and no special things to do

    // rogers.com.md: rogers.com
    // - skipping provider with status OK and no special things to do

    // t-online.md: t-online.de, magenta.de
    static ref P_T_ONLINE: Provider = Provider {
        status: Status::PREPARATION,
        before_login_hint: "To use Delta Chat with a T-Online email address, you need to create an app password in the web interface.",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/t-online",
        server: vec![
        ],
    };

    // testrun.md: testrun.org
    static ref P_TESTRUN: Provider = Provider {
        status: Status::OK,
        before_login_hint: "",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/testrun",
        server: vec![
            Server { protocol: IMAP, socket: SSL, hostname: "testrun.org", port: 993, username_pattern: EMAIL },
            Server { protocol: IMAP, socket: STARTTLS, hostname: "testrun.org", port: 143, username_pattern: EMAIL },
            Server { protocol: SMTP, socket: STARTTLS, hostname: "testrun.org", port: 587, username_pattern: EMAIL },
        ],
    };

    // tiscali.it.md: tiscali.it
    static ref P_TISCALI_IT: Provider = Provider {
        status: Status::OK,
        before_login_hint: "",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/tiscali-it",
        server: vec![
            Server { protocol: IMAP, socket: SSL, hostname: "imap.tiscali.it", port: 993, username_pattern: EMAIL },
            Server { protocol: SMTP, socket: SSL, hostname: "smtp.tiscali.it", port: 465, username_pattern: EMAIL },
        ],
    };

    // ukr.net.md: ukr.net
    // - skipping provider with status OK and no special things to do

    // vfemail.md: vfemail.net
    // - skipping provider with status OK and no special things to do

    // web.de.md: web.de, email.de, flirt.ms, hallo.ms, kuss.ms, love.ms, magic.ms, singles.ms, cool.ms, kanzler.ms, okay.ms, party.ms, pop.ms, stars.ms, techno.ms, clever.ms, deutschland.ms, genial.ms, ich.ms, online.ms, smart.ms, wichtig.ms, action.ms, fussball.ms, joker.ms, planet.ms, power.ms
    static ref P_WEB_DE: Provider = Provider {
        status: Status::PREPARATION,
        before_login_hint: "You must allow IMAP access to your account before you can login.",
        after_login_hint: "Note: if you have your web.de spam settings too strict, you won't receive contact requests from new people. If you want to receive contact requests, you should disable the \"3-Wege-Spamschutz\" in the web.de settings.  Read how: https://hilfe.web.de/email/spam-und-viren/spamschutz-einstellungen.html",
        overview_page: "https://providers.delta.chat/web-de",
        server: vec![
            Server { protocol: IMAP, socket: SSL, hostname: "imap.web.de", port: 993, username_pattern: EMAILLOCALPART },
            Server { protocol: IMAP, socket: STARTTLS, hostname: "imap.web.de", port: 143, username_pattern: EMAILLOCALPART },
            Server { protocol: SMTP, socket: STARTTLS, hostname: "smtp.web.de", port: 587, username_pattern: EMAILLOCALPART },
        ],
    };

    // yahoo.md: yahoo.com, yahoo.de, yahoo.it, yahoo.fr, yahoo.es, yahoo.se, yahoo.co.uk, yahoo.co.nz, yahoo.com.au, yahoo.com.ar, yahoo.com.br, yahoo.com.mx, ymail.com, rocketmail.com, yahoodns.net
    static ref P_YAHOO: Provider = Provider {
        status: Status::PREPARATION,
        before_login_hint: "To use Delta Chat with your Yahoo email address you have to allow \"less secure apps\" in the Yahoo webinterface.",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/yahoo",
        server: vec![
            Server { protocol: IMAP, socket: SSL, hostname: "imap.mail.yahoo.com", port: 993, username_pattern: EMAIL },
            Server { protocol: SMTP, socket: SSL, hostname: "smtp.mail.yahoo.com", port: 465, username_pattern: EMAIL },
        ],
    };

    // yandex.ru.md: yandex.ru, yandex.com
    static ref P_YANDEX_RU: Provider = Provider {
        status: Status::PREPARATION,
        before_login_hint: "For Yandex accounts, you have to set IMAP protocol option turned on.",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/yandex-ru",
        server: vec![
        ],
    };

    // ziggo.nl.md: ziggo.nl
    static ref P_ZIGGO_NL: Provider = Provider {
        status: Status::OK,
        before_login_hint: "",
        after_login_hint: "",
        overview_page: "https://providers.delta.chat/ziggo-nl",
        server: vec![
            Server { protocol: IMAP, socket: SSL, hostname: "imap.ziggo.nl", port: 993, username_pattern: EMAIL },
            Server { protocol: SMTP, socket: STARTTLS, hostname: "smtp.ziggo.nl", port: 587, username_pattern: EMAIL },
        ],
    };

    pub static ref PROVIDER_DATA: HashMap<&'static str, &'static Provider> = [
        ("aktivix.org", &*P_AKTIVIX_ORG),
        ("aol.com", &*P_AOL),
        ("autistici.org", &*P_AUTISTICI_ORG),
        ("bluewin.ch", &*P_BLUEWIN_CH),
        ("example.com", &*P_EXAMPLE_COM),
        ("example.org", &*P_EXAMPLE_COM),
        ("fastmail.com", &*P_FASTMAIL),
        ("freenet.de", &*P_FREENET_DE),
        ("gmail.com", &*P_GMAIL),
        ("googlemail.com", &*P_GMAIL),
        ("gmx.net", &*P_GMX_NET),
        ("gmx.de", &*P_GMX_NET),
        ("gmx.at", &*P_GMX_NET),
        ("gmx.ch", &*P_GMX_NET),
        ("gmx.org", &*P_GMX_NET),
        ("gmx.eu", &*P_GMX_NET),
        ("gmx.info", &*P_GMX_NET),
        ("gmx.biz", &*P_GMX_NET),
        ("gmx.com", &*P_GMX_NET),
        ("icloud.com", &*P_ICLOUD),
        ("me.com", &*P_ICLOUD),
        ("mac.com", &*P_ICLOUD),
        ("nauta.cu", &*P_NAUTA_CU),
        ("hotmail.com", &*P_OUTLOOK_COM),
        ("outlook.com", &*P_OUTLOOK_COM),
        ("office365.com", &*P_OUTLOOK_COM),
        ("outlook.com.tr", &*P_OUTLOOK_COM),
        ("live.com", &*P_OUTLOOK_COM),
        ("posteo.de", &*P_POSTEO),
        ("protonmail.com", &*P_PROTONMAIL),
        ("protonmail.ch", &*P_PROTONMAIL),
        ("t-online.de", &*P_T_ONLINE),
        ("magenta.de", &*P_T_ONLINE),
        ("testrun.org", &*P_TESTRUN),
        ("tiscali.it", &*P_TISCALI_IT),
        ("web.de", &*P_WEB_DE),
        ("email.de", &*P_WEB_DE),
        ("flirt.ms", &*P_WEB_DE),
        ("hallo.ms", &*P_WEB_DE),
        ("kuss.ms", &*P_WEB_DE),
        ("love.ms", &*P_WEB_DE),
        ("magic.ms", &*P_WEB_DE),
        ("singles.ms", &*P_WEB_DE),
        ("cool.ms", &*P_WEB_DE),
        ("kanzler.ms", &*P_WEB_DE),
        ("okay.ms", &*P_WEB_DE),
        ("party.ms", &*P_WEB_DE),
        ("pop.ms", &*P_WEB_DE),
        ("stars.ms", &*P_WEB_DE),
        ("techno.ms", &*P_WEB_DE),
        ("clever.ms", &*P_WEB_DE),
        ("deutschland.ms", &*P_WEB_DE),
        ("genial.ms", &*P_WEB_DE),
        ("ich.ms", &*P_WEB_DE),
        ("online.ms", &*P_WEB_DE),
        ("smart.ms", &*P_WEB_DE),
        ("wichtig.ms", &*P_WEB_DE),
        ("action.ms", &*P_WEB_DE),
        ("fussball.ms", &*P_WEB_DE),
        ("joker.ms", &*P_WEB_DE),
        ("planet.ms", &*P_WEB_DE),
        ("power.ms", &*P_WEB_DE),
        ("yahoo.com", &*P_YAHOO),
        ("yahoo.de", &*P_YAHOO),
        ("yahoo.it", &*P_YAHOO),
        ("yahoo.fr", &*P_YAHOO),
        ("yahoo.es", &*P_YAHOO),
        ("yahoo.se", &*P_YAHOO),
        ("yahoo.co.uk", &*P_YAHOO),
        ("yahoo.co.nz", &*P_YAHOO),
        ("yahoo.com.au", &*P_YAHOO),
        ("yahoo.com.ar", &*P_YAHOO),
        ("yahoo.com.br", &*P_YAHOO),
        ("yahoo.com.mx", &*P_YAHOO),
        ("ymail.com", &*P_YAHOO),
        ("rocketmail.com", &*P_YAHOO),
        ("yahoodns.net", &*P_YAHOO),
        ("yandex.ru", &*P_YANDEX_RU),
        ("yandex.com", &*P_YANDEX_RU),
        ("ziggo.nl", &*P_ZIGGO_NL),
    ].iter().copied().collect();
}
